/*
// $Id: //guest/julian_hyde/mondrian/src/main/mondrian/web/servlet/MDXQueryServlet.java#1 $
// (C) Copyright 2002 Kana Software, Inc.
// This software is subject to the terms of the Common Public License
// Agreement, available at the following URL:
// http://www.opensource.org/licenses/cpl.html.
// (C) Copyright 2002 Kana Software, Inc. and others.
// All Rights Reserved.
// You must accept the terms of that agreement to use this software.
//
// Sean McCullough, 13 February, 2002, 10:25 PM
*/

package mondrian.web.servlet;		

import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import mondrian.olap.*;
import mondrian.rolap.RolapConnection;
import mondrian.xom.XOMUtil;

/** 
 * <code>MDXQueryServlet</code> is a servlet which receives MDX queries,
 * executes them, and formats the results in an HTML table.
 *
 * @author  Sean McCullough
 * @since 13 February, 2002
 * @version $Id: //guest/julian_hyde/mondrian/src/main/mondrian/web/servlet/MDXQueryServlet.java#1 $
 */
public class MDXQueryServlet extends HttpServlet {
    String connectString;
    
    /** Initializes the servlet.
    */	
    public void init(ServletConfig config) throws ServletException {
	super.init(config);
	String resourceURL = config.getInitParameter("resourceURL");
	System.getProperties().put("mondrian.resourceURL", resourceURL);
	mondrian.olap.Util.setThreadRes(MondrianResource.instance());
	connectString = config.getInitParameter("connectString");
	String jdbcDrivers = config.getInitParameter("jdbcDrivers");
	StringTokenizer tok = new java.util.StringTokenizer(jdbcDrivers, ",");
	while (tok.hasMoreTokens()) {
	    String jdbcDriver = tok.nextToken();
	    try {
		Class.forName(jdbcDriver);
	    } catch (ClassNotFoundException e) {
		e.printStackTrace();
	    }
	}
    }

    /** Destroys the servlet.
    */	
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
	String queryString = request.getParameter("queryString");
	request.setAttribute("queryString", queryString);
	String resultString = "result";
	mondrian.olap.Connection mdxConnection = null;
	StringBuffer html = new StringBuffer();

	//execute the query
	try {
	    boolean do_debug_parse = false;
	    mdxConnection = DriverManager.getConnection(connectString, null, false);
	    Parser parser = new Parser();
	    Query q = parser.parseQuery(mdxConnection,queryString,do_debug_parse);
	    Result result = mdxConnection.execute(q);
	    
	    Position slicers[] = result.getSlicerAxis().positions;
	    html.append("<table class='resulttable' cellspacing=1 border=0><tr><td nowrap class='slicer'>");
	    for (int i=0; i<slicers.length; i++) {
		Position position = slicers[i];		       
		for (int j = 0; j < position.members.length; j++) {
		    Member member = position.members[j];
		    html.append(member.getUniqueName());
		}
	    }
	    html.append("&nbsp;</td>");

	    Position columns[] = result.getAxes()[0].positions;
	    for (int i=0; i<columns.length; i++) {
		Position position = columns[i];		       
		for (int j = 0; j < position.members.length; j++) {
		    Member member = position.members[j];
		    html.append("<td nowrap class='columnheading'>" + member.getUniqueName() + "</td>");
		}
	    }
	    html.append("</tr>");
			
	    if (result.getAxes().length > 1) {
		for (int i=0; i<result.getAxes()[1].positions.length; i++) {
		    //TODO: fix this to iterate over multiple members if they exist for this position
		    Member m = result.getAxes()[1].positions[i].members[0];
		    html.append("<tr><td nowrap class='rowheading'>" + m.getUniqueName() + "</td>");
		    for (int j=0; j<result.getAxes()[0].positions.length; j++) {
			Cell cell = result.getCell(new int[]{j,i});
			html.append("<td class='cell'>" + cell.getFormattedValue() + "</td>");
		    }
		    html.append("</tr>");
		}
	    } else {
		for (int i=0; i<result.getAxes()[0].positions.length; i++) {
		    Cell cell = result.getCell(new int[]{i});
		    html.append("<tr><td></td><td class='cell'>" + cell.getFormattedValue() + "</td></tr>");
		}
	    }

	    html.append("</table>");
	} catch (Throwable e) {
	    html.append("Error: " + e.getMessage());
	    e.printStackTrace();
	} finally {
	    if (mdxConnection != null) {
		mdxConnection.close();
	    }
	}

	request.setAttribute("result", html.toString());
	getServletContext().getRequestDispatcher("/index.jsp").include(request, response);
    } 
	
    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
	processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
	processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
	return "Process an MDX query and return the result formatted as an HTML table";
    }

}
